"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useEnhancedReducer = exports.getAction = exports.useKeyActionTypes = exports.useKeyInitialState = void 0;

var _react = require("react");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var useKeyInitialState = {
  keyCode: null,
  keyName: null,
  isEscape: false,
  isEnter: false,
  isSpace: false,
  isArrows: false,
  isRightArrow: false,
  isLeftArrow: false,
  isUpArrow: false,
  isDownArrow: false,
  isTab: false,
  isWithShift: false,
  isWithCtrl: false,
  isNumber: false,
  isCaps: false,
  isSmall: false,
  isSpecialCharacter: false
};
exports.useKeyInitialState = useKeyInitialState;
var useKeyActionTypes = {
  ENTER_KEY: 'ENTER_KEY',
  ESCAPE_KEY: 'ESCAPE_KEY',
  RESET_CAPTURES: 'RESET_CAPTURES',
  CAPS_ALPHABET: 'CAPS',
  SMALL_ALPHABET: 'SMALL',
  NUMBER: 'NUMBER',
  SPACE: 'SPACE',
  ARROWS: 'ARROWS'
};
exports.useKeyActionTypes = useKeyActionTypes;
var arrowKeysMapper = {
  24: 'UpArrow',
  25: 'DownArrow',
  26: 'RightArrow',
  27: 'LeftArrow'
};
var keyCodeMapper = {
  13: useKeyActionTypes.ENTER_KEY,
  27: useKeyActionTypes.ESCAPE_KEY,
  32: useKeyActionTypes.Error
};

var getArrowKeysPayload = function getArrowKeysPayload(keyCode) {
  return _defineProperty({}, "is".concat(arrowKeysMapper[keyCode]), true);
};

var isCapitalLetterPressed = function isCapitalLetterPressed(key) {
  return /^[A-Z]$/.test(key);
};

var isSmallLetterPressed = function isSmallLetterPressed(key) {
  return /^[a-z]$/.test(key);
};
/**
 * Returns the action type for the key pressed
 * @param {KeyboardEvent} eventDetails keyboard event object
 * @return {String}  action type
 */


var getAction = function getAction(eventDetails) {
  if (!eventDetails) {
    throw new Error('Event called with no details');
  }

  if (keyCodeMapper[eventDetails.keyCode]) {
    return {
      type: useKeyActionTypes.SPACE
    };
  }

  if (keyCodeMapper[eventDetails.keyCode]) return {
    type: keyCodeMapper[eventDetails.keyCode]
  };

  if (isCapitalLetterPressed(eventDetails.key)) {
    return {
      type: useKeyActionTypes.CAPS_ALPHABET
    };
  }

  if (isSmallLetterPressed(eventDetails.key)) {
    return {
      type: useKeyActionTypes.SMALL_ALPHABET
    };
  }

  if (eventDetails.keyCode >= 48 && eventDetails.keyCode <= 57) {
    return {
      type: useKeyActionTypes.NUMBER
    };
  }

  if (eventDetails.keyCode >= 24 && eventDetails.keyCode <= 27) {
    return {
      type: useKeyActionTypes.ARROWS,
      payload: getArrowKeysPayload(eventDetails.keyCode)
    };
  }

  return {
    type: 'SOME_OTHER_KEY'
  };
};

exports.getAction = getAction;

var useEnhancedReducer = function useEnhancedReducer(reducer) {
  return (0, _react.useCallback)(function (state, action) {
    return reducer(state, action);
  }, [reducer]);
}; // {
//   payload: event,
//   type: getActionType(event)
// }


exports.useEnhancedReducer = useEnhancedReducer;